@@ -24,7 +24,6 @@ const Dialog = imports.ui.dialog;
 const Gettext = imports.gettext.domain('desktop-icons');
 const ModalDialog = imports.ui.modalDialog;
 const ShellEntry = imports.ui.shellEntry;
-const Tweener = imports.ui.tweener;
 
 const ExtensionUtils = imports.misc.extensionUtils;
 const Me = ExtensionUtils.getCurrentExtension();
@@ -34,17 +33,15 @@ const Config = imports.misc.config;
 
 const _ = Gettext.gettext;
 
-const DIALOG_GROW_TIME = 0.1;
+const DIALOG_GROW_TIME = 100;
 
-const DIALOG_INIT_PROPS = { styleClass: 'create-folder-dialog' };
-
-let CreateFolderDialogClass = class CreateFolderDialog extends ModalDialog.ModalDialog {
+var CreateFolderDialog = GObject.registerClass({
+    GTypeName: 'DesktopIcons_CreateFolderDialog',
+    Signals: { 'response': { param_types: [GObject.TYPE_STRING] } }
+}, class CreateFolderDialog extends ModalDialog.ModalDialog {
     _init() {
-        super._init(DIALOG_INIT_PROPS);
-        this._buildLayout();
-    }
+        super._init({ styleClass: 'create-folder-dialog' });
 
-    _buildLayout() {
         let label = new St.Label({ style_class: 'create-folder-dialog-label',
                                    text: _('New folder name') });
         this.contentLayout.add(label, { x_align: St.Align.START });
@@ -84,15 +81,16 @@ let CreateFolderDialogClass = class CreateFolderDialog extends ModalDialog.Modal
             let [errorBoxMinHeight, errorBoxNaturalHeight] = this._errorBox.get_preferred_height(-1);
             let parentActor = this._errorBox.get_parent();
 
-            Tweener.addTween(parentActor,
-                             { height: parentActor.height + errorBoxNaturalHeight,
-                               time: DIALOG_GROW_TIME,
-                               transition: 'easeOutQuad',
-                               onComplete: () => {
-                                   parentActor.set_height(-1);
-                                   this._errorBox.show();
-                               }
-                             });
+            parentActor.remove_all_transitions();
+            parentActor.ease_property(
+                'height', parentActor.height + errorBoxNaturalHeight, {
+                duration: DIALOG_GROW_TIME,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                onComplete: () => {
+                    parentActor.set_height(-1);
+                    this._errorBox.show();
+                }
+            });
         }
     }
 
@@ -101,16 +99,17 @@ let CreateFolderDialogClass = class CreateFolderDialog extends ModalDialog.Modal
             let [errorBoxMinHeight, errorBoxNaturalHeight] = this._errorBox.get_preferred_height(-1);
             let parentActor = this._errorBox.get_parent();
 
-            Tweener.addTween(parentActor,
-                             { height: parentActor.height - errorBoxNaturalHeight,
-                               time: DIALOG_GROW_TIME,
-                               transition: 'easeOutQuad',
-                               onComplete: () => {
-                                   parentActor.set_height(-1);
-                                   this._errorBox.hide();
-                                   this._errorMessage.set_text('');
-                               }
-                             });
+            parentActor.remove_all_transitions();
+            parentActor.ease_property(
+                'height', parentActor.height - errorBoxNaturalHeight, {
+                duration: DIALOG_GROW_TIME,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                onComplete: () => {
+                    parentActor.set_height(-1);
+                    this._errorBox.hide();
+                    this._errorMessage.set_text('');
+                }
+            });
         }
     }
 
@@ -161,21 +160,4 @@ let CreateFolderDialogClass = class CreateFolderDialog extends ModalDialog.Modal
 
         this._createButton.reactive = is_valid;
     }
-};
-
-var CreateFolderDialog;
-
-if (ExtensionUtils.versionCheck(['3.28', '3.30', '3.32'], Config.PACKAGE_VERSION)) {
-    CreateFolderDialog = class extends CreateFolderDialogClass {
-        constructor() {
-            super(DIALOG_INIT_PROPS);
-            this._buildLayout();
-        }
-    };
-    Signals.addSignalMethods(CreateFolderDialogClass.prototype);
-} else {
-    CreateFolderDialog = GObject.registerClass({
-        GTypeName: 'DesktopIcons_CreateFolderDialog',
-        Signals: { 'response': { param_types: [GObject.TYPE_STRING] } }
-    }, CreateFolderDialogClass);
-}
+});
