@@ -24,8 +24,6 @@ const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Shell = imports.gi.Shell;
 
-const Signals = imports.signals;
-
 const Layout = imports.ui.layout;
 const Main = imports.ui.main;
 const BoxPointer = imports.ui.boxpointer;
@@ -68,9 +66,10 @@ var Placeholder = GObject.registerClass({
 }, class Placeholder extends St.Bin {
 });
 
-var DesktopGrid = class {
-
-    constructor(bgManager) {
+var DesktopGrid = GObject.registerClass({
+    GTypeName: 'DesktopIcons_DesktopGrid',
+}, class DesktopGrid extends St.Widget {
+    _init(bgManager) {
         this._bgManager = bgManager;
 
         this._fileItemHandlers = new Map();
@@ -87,12 +86,12 @@ var DesktopGrid = class {
             y_align: Clutter.BinAlignment.FIXED
         });
 
-        this.actor = new St.Widget({
+        super._init({
             layout_manager: this._actorLayout
         });
-        this.actor._delegate = this;
+        this._delegate = this;
 
-        this.actor.connect('style-changed', () => {
+        this.connect('style-changed', () => {
             if (this.isFirst)
                 Extension.desktopManager.scheduleReLayoutChildren();
         });
@@ -106,20 +105,18 @@ var DesktopGrid = class {
             can_focus: true,
             opacity: 255
         });
-        this.actor.add_child(this._grid);
+        this.add_child(this._grid);
 
-        this._menuManager = new PopupMenu.PopupMenuManager(this.actor);
+        this._menuManager = new PopupMenu.PopupMenuManager(this);
 
-        this._bgManager._container.add_child(this.actor);
-
-        this.actor.connect('destroy', () => this._onDestroy());
+        this._bgManager._container.add_child(this);
 
         let monitorIndex = bgManager._monitorIndex;
         this._monitorConstraint = new Layout.MonitorConstraint({
             index: monitorIndex,
             work_area: true
         });
-        this.actor.add_constraint(this._monitorConstraint);
+        this.add_constraint(this._monitorConstraint);
 
         this._addDesktopBackgroundMenu();
 
@@ -132,7 +129,7 @@ var DesktopGrid = class {
 
         this._grid.connect('allocation-changed', () => Extension.desktopManager.scheduleReLayoutChildren());
 
-        let themeNodeBase = this.actor.get_theme_node();
+        let themeNodeBase = this.get_theme_node();
         let themeContext = St.ThemeContext.get_for_stage(global.stage);
         let themeNode = St.ThemeNode.new(themeContext, null,
             themeNodeBase.get_theme(), themeNodeBase.get_element_type(),
@@ -147,6 +144,8 @@ var DesktopGrid = class {
                              themeNode.get_border_width(St.Side.TOP) +
                              themeNode.get_border_width(St.Side.BOTTOM) +
                              themeNode.get_vertical_padding();
+
+        this.connect('destroy', () => this._onDestroy());
     }
 
     _onKeyPress(actor, event) {
@@ -201,12 +200,12 @@ var DesktopGrid = class {
             this._bgDestroyedId = this._bgManager.backgroundActor.connect('destroy',
                 () => this._backgroundDestroyed());
         } else {
-            this.actor.destroy();
+            this.destroy();
         }
     }
 
     _onDestroy() {
-        if (this._bgDestroyedId && this._bgManager.backgroundActor != null)
+        if (this._bgDestroyedId && this._bgManager.backgroundActor)
             this._bgManager.backgroundActor.disconnect(this._bgDestroyedId);
         this._bgDestroyedId = 0;
         this._bgManager = null;
@@ -329,12 +328,8 @@ var DesktopGrid = class {
     }
 
     _syncUndoRedo() {
-        let useItemActors = !ExtensionUtils.versionCheck(['3.28', '3.30', '3.32'], Config.PACKAGE_VERSION);
-        let undoActor = useItemActors ? this._undoMenuItem : this._undoMenuItem.actor;
-        let redoActor = useItemActors ? this._redoMenuItem : this._redoMenuItem.actor;
-
-        undoActor.visible = DBusUtils.NautilusFileOperationsProxy.UndoStatus == UndoStatus.UNDO;
-        redoActor.visible = DBusUtils.NautilusFileOperationsProxy.UndoStatus == UndoStatus.REDO;
+        this._undoMenuItem.visible = DBusUtils.NautilusFileOperationsProxy.UndoStatus == UndoStatus.UNDO;
+        this._redoMenuItem.visible = DBusUtils.NautilusFileOperationsProxy.UndoStatus == UndoStatus.REDO;
     }
 
     _undoStatusChanged(proxy, properties, test) {
@@ -389,7 +384,7 @@ var DesktopGrid = class {
 
     _openMenu(x, y) {
         Main.layoutManager.setDummyCursorGeometry(x, y, 0, 0);
-        this.actor._desktopBackgroundMenu.open(BoxPointer.PopupAnimation.NONE);
+        this._desktopBackgroundMenu.open(BoxPointer.PopupAnimation.NONE);
         /* Since the handler is in the press event it needs to ignore the release event
          * to not immediately close the menu on release
          */
@@ -398,11 +393,17 @@ var DesktopGrid = class {
 
     _addFileItemTo(fileItem, column, row, coordinatesAction) {
         let placeholder = this.layout.get_child_at(column, row);
-        placeholder.child = fileItem.actor;
+        placeholder.child = fileItem;
         this._fileItems.push(fileItem);
-        let selectedId = fileItem.connect('selected', this._onFileItemSelected.bind(this));
-        let renameId = fileItem.connect('rename-clicked', this._onRenameClicked.bind(this));
-        this._fileItemHandlers.set(fileItem, [selectedId, renameId]);
+        let fileItemHandler = {};
+        fileItemHandler.selectedId = fileItem.connect('selected',
+            this._onFileItemSelected.bind(this));
+        fileItemHandler.renameId = fileItem.connect('rename-clicked',
+            this._onRenameClicked.bind(this));
+        fileItemHandler.destroyId = fileItem.connect('destroy', () => {
+            this._fileItemHandlers.delete(fileItem);
+        });
+        this._fileItemHandlers.set(fileItem, fileItemHandler);
 
         /* If this file is new in the Desktop and hasn't yet
          * fixed coordinates, store the new possition to ensure
@@ -475,9 +476,8 @@ var DesktopGrid = class {
         let [column, row] = this._getPosOfFileItem(fileItem);
         let placeholder = this.layout.get_child_at(column, row);
         placeholder.child = null;
-        let [selectedId, renameId] = this._fileItemHandlers.get(fileItem);
-        fileItem.disconnect(selectedId);
-        fileItem.disconnect(renameId);
+        let fileItemHandler = this._fileItemHandlers.get(fileItem);
+        Object.values(fileItemHandler).forEach(id => fileItem.disconnect(id));
         this._fileItemHandlers.delete(fileItem);
     }
 
@@ -533,12 +533,12 @@ var DesktopGrid = class {
     }
 
     _addDesktopBackgroundMenu() {
-        this.actor._desktopBackgroundMenu = this._createDesktopBackgroundMenu();
-        this._menuManager.addMenu(this.actor._desktopBackgroundMenu);
+        this._desktopBackgroundMenu = this._createDesktopBackgroundMenu();
+        this._menuManager.addMenu(this._desktopBackgroundMenu);
 
-        this.actor.connect('destroy', () => {
-            this.actor._desktopBackgroundMenu.destroy();
-            this.actor._desktopBackgroundMenu = null;
+        this.connect('destroy', () => {
+            this._desktopBackgroundMenu.destroy();
+            this._desktopBackgroundMenu = null;
         });
     }
 
@@ -559,7 +559,7 @@ var DesktopGrid = class {
     acceptDrop(source, actor, x, y, time) {
         /* Coordinates are relative to the grid, we want to transform them to
          * absolute coordinates to work across monitors */
-        let [gridX, gridY] = this.actor.get_transformed_position();
+        let [gridX, gridY] = this.get_transformed_position();
         let [absoluteX, absoluteY] = [x + gridX, y + gridY];
         return Extension.desktopManager.acceptDrop(absoluteX, absoluteY);
     }
@@ -576,7 +576,7 @@ var DesktopGrid = class {
         for (column = 0; column < maxColumns; column++) {
             for (row = 0; row < maxRows; row++) {
                 let item = this.layout.get_child_at(column, row);
-                if (item.child && item.child._delegate.file.equal(itemToFind.file)) {
+                if (item.child && item.child.file.equal(itemToFind.file)) {
                     found = true;
                     break;
                 }
@@ -615,7 +615,7 @@ var DesktopGrid = class {
         renamePopup.connect('menu-closed', () => renamePopup.destroy());
         renamePopup.open();
     }
-};
+});
 
 var RenamePopupMenuItem = GObject.registerClass(
 class RenamePopupMenuItem extends PopupMenu.PopupBaseMenuItem {
@@ -634,7 +634,6 @@ class RenamePopupMenuItem extends PopupMenu.PopupBaseMenuItem {
 
         // Entry
         this._entry = new St.Entry({
-            hint_text: _("Enter file name…"),
             x_expand: true,
             width: 200,
         });
@@ -698,7 +697,7 @@ class RenamePopupMenuItem extends PopupMenu.PopupBaseMenuItem {
 
 var RenamePopup = class RenameFolderMenu extends PopupMenu.PopupMenu {
     constructor(fileItem) {
-        super(fileItem.actor, 0.5, St.Side.TOP);
+        super(fileItem, 0.5, St.Side.TOP);
         this.actor.add_style_class_name('rename-popup');
 
         // We want to keep the item hovered while the menu is up
@@ -723,12 +722,12 @@ var RenamePopup = class RenameFolderMenu extends PopupMenu.PopupMenu {
         }
 
         // Chain our visibility and lifecycle to that of the fileItem source
-        this._fileItemMappedId = fileItem.actor.connect('notify::mapped', () => {
-            if (!fileItem.actor.mapped)
+        this._fileItemMappedId = fileItem.connect('notify::mapped', () => {
+            if (!fileItem.mapped)
                 this.close();
         });
-        fileItem.actor.connect('destroy', () => {
-            fileItem.actor.disconnect(this._fileItemMappedId);
+        fileItem.connect('destroy', () => {
+            fileItem.disconnect(this._fileItemMappedId);
             this.destroy();
         });
 
