@@ -37,12 +37,45 @@ function clamp(value, min, max) {
     return Math.max(Math.min(value, max), min);
 };
 
-function getTerminalCommand(workdir) {
+function launchTerminal(workdir) {
     let terminalSettings = new Gio.Settings({ schema_id: TERMINAL_SCHEMA });
     let exec = terminalSettings.get_string(EXEC_KEY);
-    let command = `${exec} --working-directory=${workdir}`;
+    let argv = [exec, `--working-directory=${workdir}`];
 
-    return command;
+    /* The following code has been extracted from GNOME Shell's
+     * source code in Misc.Util.trySpawn function and modified to
+     * set the working directory.
+     *
+     * https://gitlab.gnome.org/GNOME/gnome-shell/blob/gnome-3-30/js/misc/util.js
+     */
+
+    var success, pid;
+    try {
+        [success, pid] = GLib.spawn_async(workdir, argv, null,
+                                          GLib.SpawnFlags.SEARCH_PATH | GLib.SpawnFlags.DO_NOT_REAP_CHILD,
+                                          null);
+    } catch (err) {
+        /* Rewrite the error in case of ENOENT */
+        if (err.matches(GLib.SpawnError, GLib.SpawnError.NOENT)) {
+            throw new GLib.SpawnError({ code: GLib.SpawnError.NOENT,
+                                        message: _("Command not found") });
+        } else if (err instanceof GLib.Error) {
+            // The exception from gjs contains an error string like:
+            //   Error invoking GLib.spawn_command_line_async: Failed to
+            //   execute child process "foo" (No such file or directory)
+            // We are only interested in the part in the parentheses. (And
+            // we can't pattern match the text, since it gets localized.)
+            let message = err.message.replace(/.*\((.+)\)/, '$1');
+            throw new (err.constructor)({ code: err.code,
+                                          message: message });
+        } else {
+            throw err;
+        }
+    }
+    // Dummy child watch; we don't want to double-fork internally
+    // because then we lose the parent-child relationship, which
+    // can break polkit.  See https://bugzilla.redhat.com//show_bug.cgi?id=819275
+    GLib.child_watch_add(GLib.PRIORITY_DEFAULT, pid, () => {});
 }
 
 function distanceBetweenPoints(x, y, x2, y2) {
