@@ -94,6 +94,10 @@ var DesktopManager = GObject.registerClass({
             if (key == 'show-hidden')
                 this._recreateDesktopIcons();
         });
+        this.nautilusSettingsId = Prefs.nautilusSettings.connect('changed', (obj, key) => {
+            if (key == 'show-image-thumbnails')
+                this._recreateDesktopIcons();
+        });
 
         this._selection = new Set();
         this._currentSelection = new Set();
@@ -178,6 +182,7 @@ var DesktopManager = GObject.registerClass({
     }
 
     _recreateDesktopIcons() {
+        this.clearSelection();
         this._destroyDesktopIcons();
         this._addDesktopIcons();
     }
@@ -186,10 +191,10 @@ var DesktopManager = GObject.registerClass({
         let first_element = true;
         forEachBackgroundManager(bgManager => {
             let newGrid = new DesktopGrid.DesktopGrid(bgManager);
-            newGrid.actor.connect('destroy', (actor) => {
+            newGrid.connect('destroy', (actor) => {
                 // if a grid loses its actor, remove it from the grid list
                 for (let grid in this._desktopGrids)
-                    if (this._desktopGrids[grid].actor == actor) {
+                    if (this._desktopGrids[grid] == actor) {
                         delete this._desktopGrids[grid];
                         break;
                     }
@@ -203,7 +208,7 @@ var DesktopManager = GObject.registerClass({
     }
 
     _destroyDesktopIcons() {
-        Object.values(this._desktopGrids).forEach(grid => grid.actor.destroy());
+        Object.values(this._desktopGrids).forEach(grid => grid.destroy());
         this._desktopGrids = {};
     }
 
@@ -218,8 +223,9 @@ var DesktopManager = GObject.registerClass({
     }
 
     async _scanFiles() {
-        for (let [fileItem, id] of this._fileItemHandlers)
-            fileItem.disconnect(id);
+        for (let [fileItem, fileItemHandler] of this._fileItemHandlers)
+            Object.values(fileItemHandler).forEach(id => fileItem.disconnect(id));
+
         this._fileItemHandlers = new Map();
 
         if (!this._unixMode) {
@@ -236,10 +242,13 @@ var DesktopManager = GObject.registerClass({
             for (let [file, info, extra] of await this._enumerateDesktop()) {
                 let fileItem = new FileItem.FileItem(file, info, extra);
                 tmpFileItems.set(fileItem.file.get_uri(), fileItem);
-                let id = fileItem.connect('selected',
-                                          this._onFileItemSelected.bind(this));
-
-                this._fileItemHandlers.set(fileItem, id);
+                let fileItemHandler = {}
+                fileItemHandler.selectedId = fileItem.connect('selected',
+                    this._onFileItemSelected.bind(this));
+                fileItemHandler.destroyId = fileItem.connect('destroy', () => {
+                    this._fileItemHandlers.delete(fileItem);
+                });
+                this._fileItemHandlers.set(fileItem, fileItemHandler);
             }
             this._fileItems = tmpFileItems;
         } catch (e) {
@@ -332,9 +341,7 @@ var DesktopManager = GObject.registerClass({
         } = Gio.FileMonitorEvent;
 
         let fileUri = file.get_uri();
-        let fileItem = null;
-        if (this._fileItems.has(fileUri))
-            fileItem = this._fileItems.get(fileUri);
+        let fileItem = this._fileItems.get(fileUri);
         switch(eventType) {
             case RENAMED:
                 this._fileItems.delete(fileUri);
@@ -345,7 +352,7 @@ var DesktopManager = GObject.registerClass({
             case ATTRIBUTE_CHANGED:
                 /* a file changed, rather than the desktop itself */
                 let desktopDir = DesktopIconsUtil.getDesktopDir();
-                if (file.get_uri() != desktopDir.get_uri()) {
+                if (fileItem && file.get_uri() != desktopDir.get_uri()) {
                     fileItem.onAttributeChanged();
                     return;
                 }
@@ -419,11 +426,11 @@ var DesktopManager = GObject.registerClass({
 
         for (let fileItem of this._selection) {
             let clone = new Clutter.Clone({
-                source: fileItem.actor,
+                source: fileItem,
                 reactive: false
             });
-            clone.x = fileItem.actor.get_transformed_position()[0];
-            clone.y = fileItem.actor.get_transformed_position()[1];
+            clone.x = fileItem.get_transformed_position()[0];
+            clone.y = fileItem.get_transformed_position()[1];
             this._draggableContainer.add_child(clone);
         }
 
@@ -520,7 +527,7 @@ var DesktopManager = GObject.registerClass({
          * them.
          */
         for (let item of this._selection) {
-            let [itemX, itemY] = item.actor.get_transformed_position();
+            let [itemX, itemY] = item.get_transformed_position();
             let monitorIndex = findMonitorIndexForPos(itemX, itemY);
             savedCoordinates.set(item, [itemX, itemY]);
             this._desktopGrids[monitorIndex].removeFileItem(item);
@@ -722,6 +729,9 @@ var DesktopManager = GObject.registerClass({
         if (this.gtkSettingsId)
             Prefs.gtkSettings.disconnect(this.gtkSettingsId);
         this.gtkSettingsId = 0;
+        if (this.nautilusSettingsId)
+            Prefs.nautilusSettings.disconnect(this.nautilusSettingsId);
+        this.nautilusSettingsId = 0;
 
         if (this._layoutChildrenId)
             GLib.source_remove(this._layoutChildrenId);
@@ -747,7 +757,7 @@ var DesktopManager = GObject.registerClass({
         if (this._queryFileInfoCancellable)
             this._queryFileInfoCancellable.cancel();
 
-        Object.values(this._desktopGrids).forEach(grid => grid.actor.destroy());
+        Object.values(this._desktopGrids).forEach(grid => grid.destroy());
         this._desktopGrids = {}
     }
 });
