@@ -20,6 +20,7 @@ const Gtk = imports.gi.Gtk;
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const St = imports.gi.St;
 const Pango = imports.gi.Pango;
 const Meta = imports.gi.Meta;
@@ -28,7 +29,6 @@ const Cogl = imports.gi.Cogl;
 const GnomeDesktop = imports.gi.GnomeDesktop;
 
 const Mainloop = imports.mainloop;
-const Signals = imports.signals;
 
 const Config = imports.misc.config;
 const Background = imports.ui.background;
@@ -57,9 +57,18 @@ var State = {
     GONE: 1,
 };
 
-var FileItem = class {
+var FileItem = GObject.registerClass({
+    GTypeName: 'DesktopIcons_FileItem',
+    Signals: {
+        'rename-clicked': {},
+        'selected': {
+            param_types: [GObject.TYPE_BOOLEAN, GObject.TYPE_BOOLEAN, GObject.TYPE_BOOLEAN]
+        }
+    }
+}, class FileItem extends St.Bin {
+    _init(file, fileInfo, fileExtra) {
+        super._init({ visible: true });
 
-    constructor(file, fileInfo, fileExtra) {
         this._fileExtra = fileExtra;
         this._loadThumbnailDataCancellable = null;
         this._thumbnailScriptWatch = 0;
@@ -76,11 +85,10 @@ var FileItem = class {
 
         this._state = State.NORMAL;
 
-        this.actor = new St.Bin({ visible: true });
-        this.actor.set_fill(true, true);
+        this.set_fill(true, true);
         let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-        this.actor._delegate = this;
-        this.actor.connect('destroy', () => this._onDestroy());
+        this._delegate = this;
+        this.connect('destroy', () => this._onDestroy());
 
         this._container = new St.BoxLayout({ reactive: true,
                                              track_hover: true,
@@ -90,7 +98,7 @@ var FileItem = class {
                                              y_expand: true,
                                              x_align: Clutter.ActorAlign.FILL,
                                              vertical: true });
-        this.actor.set_child(this._container);
+        this.set_child(this._container);
         this._icon = new St.Bin();
         this._icon.set_height(Prefs.get_icon_size() * scaleFactor);
 
@@ -112,6 +120,7 @@ var FileItem = class {
         this._container.connect('button-press-event', (actor, event) => this._onPressButton(actor, event));
         this._container.connect('motion-event', (actor, event) => this._onMotion(actor, event));
         this._container.connect('leave-event', (actor, event) => this._onLeave(actor, event));
+        this._container.connect('enter-event', (actor, event) => this._onEnter(actor, event));
         this._container.connect('button-release-event', (actor, event) => this._onReleaseButton(actor, event));
 
         /* Set the metadata and update relevant UI */
@@ -158,14 +167,12 @@ var FileItem = class {
 
     set_margins(width, height) {
         let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-        this.actor.set_width(Prefs.getDesiredWidth(scaleFactor, width));
-        this.actor.set_height(Prefs.getDesiredHeight(scaleFactor, height));
+        this.set_width(Prefs.getDesiredWidth(scaleFactor, width));
+        this.set_height(Prefs.getDesiredHeight(scaleFactor, height));
     }
 
     onAttributeChanged() {
-        if (this._isDesktopFile) {
-            this._refreshMetadataAsync(true);
-        }
+        this._refreshMetadataAsync(this._isDesktopFile);
     }
 
     _onDestroy() {
@@ -234,7 +241,7 @@ var FileItem = class {
 
         this._displayName = fileInfo.get_attribute_as_string('standard::display-name');
         this._attributeCanExecute = fileInfo.get_attribute_boolean('access::can-execute');
-        this._unixmode = fileInfo.get_attribute_uint32('unix::mode')
+        this._unixmode = fileInfo.get_attribute_uint32('unix::mode');
         this._writableByOthers = (this._unixmode & S_IWOTH) != 0;
         this._trusted = fileInfo.get_attribute_as_string('metadata::trusted') == 'true';
         this._attributeContentType = fileInfo.get_content_type();
@@ -271,7 +278,7 @@ var FileItem = class {
          * it must be a broken link.
          * https://developer.gnome.org/gio/stable/GFile.html#g-file-query-info
          */
-        this._isBrokenSymlink = this._isSymlink && this._fileType == Gio.FileType.SYMBOLIC_LINK
+        this._isBrokenSymlink = this._isSymlink && this._fileType == Gio.FileType.SYMBOLIC_LINK;
     }
 
     onFileRenamed(file) {
@@ -286,9 +293,10 @@ var FileItem = class {
         }
 
         let thumbnailFactory = GnomeDesktop.DesktopThumbnailFactory.new(GnomeDesktop.DesktopThumbnailSize.LARGE);
-        if (thumbnailFactory.can_thumbnail(this._file.get_uri(),
-                                           this._attributeContentType,
-                                           this._modifiedTime)) {
+        if ((Prefs.nautilusSettings.get_string('show-image-thumbnails') != 'never') &&
+            (thumbnailFactory.can_thumbnail(this._file.get_uri(),
+                                            this._attributeContentType,
+                                            this._modifiedTime))) {
             let thumbnail = thumbnailFactory.lookup(this._file.get_uri(), this._modifiedTime);
             if (thumbnail == null) {
                 if (!thumbnailFactory.has_valid_failed_thumbnail(this._file.get_uri(),
@@ -338,13 +346,19 @@ var FileItem = class {
                                 );
                                 let icon = new Clutter.Actor();
                                 icon.set_content(thumbnailImage);
-                                let width = Prefs.getDesiredWidth(scaleFactor, 0);
-                                let height = Prefs.get_icon_size() * scaleFactor;
-                                let aspectRatio = thumbnailPixbuf.width / thumbnailPixbuf.height;
-                                if ((width / height) > aspectRatio)
-                                    icon.set_size(height * aspectRatio, height);
-                                else
-                                    icon.set_size(width, width / aspectRatio);
+                                let containerWidth = Prefs.getDesiredWidth(scaleFactor, 0);
+                                let containerHeight = Prefs.get_icon_size() * scaleFactor;
+                                let containerAspectRatio = containerWidth / containerHeight;
+                                let iconAspectRatio = thumbnailPixbuf.width / thumbnailPixbuf.height;
+                                if (containerAspectRatio > iconAspectRatio) {
+                                    let iconWidth = containerHeight * iconAspectRatio;
+                                    icon.set_size(iconWidth, containerHeight);
+                                    icon.margin_left = icon.margin_right = Math.floor((containerWidth - iconWidth) / 2);
+                                } else {
+                                    let iconHeight = containerWidth / iconAspectRatio;
+                                    icon.set_size(containerWidth, iconHeight);
+                                    icon.margin_top = icon.margin_bottom = Math.floor((containerHeight - iconHeight) / 2);
+                                }
                                 this._icon.child = icon;
                             }
                         } catch (error) {
@@ -445,10 +459,13 @@ var FileItem = class {
             return;
         }
 
-        if (this._attributeCanExecute && !this._isDirectory && !this._isValidDesktopFile) {
-            if (this._execLine)
-                Util.spawnCommandLine(this._execLine);
-            return;
+        if (this._attributeCanExecute &&
+           !this._isDirectory &&
+           !this._isValidDesktopFile &&
+            Gio.content_type_can_be_executable(this._attributeContentType)) {
+                if (this._execLine)
+                    Util.spawnCommandLine(this._execLine);
+                return;
         }
 
         Gio.AppInfo.launch_default_for_uri_async(this.file.get_uri(),
@@ -572,7 +589,7 @@ var FileItem = class {
             'rgba(' + rgba.red * 255 + ', ' + rgba.green * 255 + ', ' + rgba.blue * 255 + ', 0.8)';
 
         return 'background-color: ' + background_color + ';' +
-               'border-color: ' + border_color + ';'
+               'border-color: ' + border_color + ';';
     }
 
     get menu() {
@@ -594,14 +611,11 @@ var FileItem = class {
 
     _recreateMenu() {
         this._removeMenu();
-        let popupManagerArgs = ExtensionUtils.versionCheck(['3.28', '3.30', '3.32'], Config.PACKAGE_VERSION) ?
-            { actor: this.actor } : this.actor;
-
-        this._menuManager = new PopupMenu.PopupMenuManager(popupManagerArgs);
+        this._menuManager = new PopupMenu.PopupMenuManager(this);
         let side = St.Side.LEFT;
         if (Clutter.get_default_text_direction() == Clutter.TextDirection.RTL)
             side = St.Side.RIGHT;
-        this._menu = new PopupMenu.PopupMenu(this.actor, 0.5, side);
+        this._menu = new PopupMenu.PopupMenu(this, 0.5, side);
         this._menu.addAction(_('Open'), () => this.doOpen());
         switch (this._fileExtra) {
         case Prefs.FileType.NONE:
@@ -689,8 +703,17 @@ var FileItem = class {
         return Clutter.EVENT_PROPAGATE;
     }
 
+    _onEnter(actor, event) {
+        if (Prefs.CLICK_POLICY_SINGLE)
+            global.display.set_cursor(Meta.Cursor.POINTING_HAND);
+        else
+            global.display.set_cursor(Meta.Cursor.DEFAULT);
+    }
+
     _onLeave(actor, event) {
         this._primaryButtonPressed = false;
+        if (Prefs.CLICK_POLICY_SINGLE)
+           global.display.set_cursor(Meta.Cursor.DEFAULT);
     }
 
     _onMotion(actor, event) {
@@ -837,5 +860,4 @@ var FileItem = class {
     acceptDrop() {
         return Extension.desktopManager.selectionDropOnFileItem(this);
     }
-};
-Signals.addSignalMethods(FileItem.prototype);
+});
